/*********************************************************
 *			AIR WAR v0.1
 *		      hacked by g0blin
 * g0blin@hackerjournal.it - http://xoomer.virgilio.it/giaki1/index.htm
 *
 *Based on software by H. Fortuna, (http://www.hsfortuna.pwp.blueyonder.co.uk/)
 * adapted as 2D game engine.
 *Sprite artwork by Ari Feldman (http://www.arifeldman.com/).
 *Sound managing code by Marteen Hofman. (not yet implemented)
 *This engine can be used for a variety of 2D games.
 *Try to transform in an Asteroid clone if you can dig it!
 *Started on July 11, 2004.
 *
 *Code history
 *
 * Jul 11, Initial adaptation from H.F. code
 * Jul 31, Content complete.
 * Aug 3,  First production milestone. Background is built and airplanes fly across the screen.
 * Aug 18, Second produtction milestone.
 * Sept 4, Content revision.
 * Sept 9, Code complete.
 *
 * 	   Rotate sprite left or right with D-pad, move foreward with X button.
 *	   Buttons are pressure sensitive.
 *	   Press Start&Select on PAD 1 to quit game.
 *
 *
 *TO DO
 *
 *sprite movement using analog stick
 *write tutorial using game engine
 *include sound & vibrations when aircraft is hit
**********************************************************/

#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2util.h>
#include <sps2regstructs.h>
#include <assert.h>
#include "PS2Defines.h"
#include "PS2Types.h"
#include "texture.h"
#include "packet.h"
#include "pad.h"
#include "vecmaths.h"

// SPRITE ROTATION USING MATRICES

// Work out how we are going to allocate SPS2 to each use:
// Here we have 4K allocated for primitives, that is right at the start of the memory
// Next, 256K allocated for the texture.
// Finally 4K allocated for the texture upload function to use for DMAing
// (The KILOBYTES macro just multiplies the number by 1024)
#define PRIMITIVEMEM_4K		0		// These macros are defined in PS2Defines.h
#define TEXTUREMEM_256K		(KILOBYTES(4))
#define TEXTUREDMAMEM_4K	(KILOBYTES(260))
#define TOTALMEMNEEDED		(KILOBYTES(264))


// Function Prototypes
void TitleScreen(void);
void GameOver(int);
void MoveSprite(PS2Sprite_t * pSP, float Delta, int Axis);
void AnimateSprite(PS2Sprite_t * pSP);
void AnimateSub(PS2Sprite_t * pSP);
void SetFire(PS2Sprite_t * pSP1, PS2Sprite_t * pSP2);
void DoControls(PS2Sprite_t * pSP1, PS2Sprite_t * pSP2);
void CheckPlayground(PS2Sprite_t * pSP1, PS2Sprite_t * pSP2);
bool Collision(PS2Sprite_t * pSP1, PS2Sprite_t * pSP2);

//globals
sps2Memory_t * pMemCached;	//pointer to allocated memory structure
int iSPS2Desc;				//the SPS2 handle
int Frame = 0,				//frame count
    Loop = 1,				//submarine loop up&down
    Fire = 0,				//fire on/off
    Explosion = 0,			//explosion on/off
    JetFire = 0,			//jet has fired
    JetDead = 0,			//remember if jet is killed
    PropFire = 0,			//prop has fired
    PropDead = 0;			//remember if prop is killed

int main(void)
{
	char * pcMemory;		//pointer to start of allocated memory
	char * pPrimitiveMem;		//pointer to the start of primitive memory
	Dn_CHCR_t chcr;			// The DMA registers we will be using.
	Dn_MADR_t madr;

	//sprites constructor section
	float p[6][2] = {{-100.0f, 50.0f}, {100.0f, 50.0f}, {-180.0f, -170.0f},
			 {-160.0f, 200.0f}, {230.0f, 170.0f}, {-45.0f, -175.0f}}; //initial position on screen
	int z[6] = {1000, 500, 10, 10, 10, 10};			   //initial z_values on screen
	int offset[4][2] = { {0,0}, {0, 162}, {64,162}, {130,162}}; //TEX64 offsets (see below)

	enum {JET, TEX96};
	enum {PROP, LAGOON, ISLAND, VULCANO, TEX64};
	enum {SUB, TEX94};
	enum {FIRE, EXPLOSION, TEX32};

	PS2Sprite_t Sprite64[TEX64],		//prop - islands
		    Sprite96[TEX96],		//jet
		    Sprite94[TEX94],		//sub
		    Sprite32[TEX32];		//fire - explosion

	//initialise the pads
	if(pad_init (PAD_0 | PAD_1, PAD_INIT_LOCK |PAD_INIT_ANALOGUE | PAD_INIT_PRESSURE) == 0)
		{
		printf("Unable to initialise the pads\n");
		pad_cleanup (PAD_0 | PAD_1);
		exit(0);
		}

	//initialise the SPS2 module
	iSPS2Desc = sps2Init();
	// Initialise the screen
	sps2UScreenInit(0);

	// Allocate the SPS2 memory
	pMemCached = sps2Allocate(TOTALMEMNEEDED,
							SPS2_MAP_BLOCK_4K |	SPS2_MAP_CACHED,
							iSPS2Desc);

	// Get a char pointer, we can use this to access memory addresses
	// relative to the start of SPS2 memory
	pcMemory = (char *)pMemCached->pvStart;

	//pointer to the start of primitive memory
	pPrimitiveMem = &pcMemory[PRIMITIVEMEM_4K];

	// Load the 256*256 bitmap into SPS2 memory
	// The final parameter says whether or not to make RGB(0,0,0)
	// transparent. Modify to make sky blue transparent instead of black (see LoadBitmap()
	// function in texture.cpp)
	if(!LoadBitmap("game1945.bmp", &pcMemory[TEXTUREMEM_256K], true))
	{
		printf("Failed to load texture.");
		return -1;
	}

	// Upload the texture from SPS2 memory into the first free
	// page of GS Memory.
	// The second parameter is the address of the texture in SPS2 memory
	// The final parameter is 4K of memory that the Upload function can use
	// for creating its own DMA chain.
	Upload(sps2UScreenGetFirstFreeGSPage(),
			&pcMemory[TEXTUREMEM_256K],
			&pcMemory[TEXTUREDMAMEM_4K]);

	TitleScreen();
	GameOver(1);

	// Configure the Sprites Structures

	// Set identity matrix to identity (for all the sprites needed)
	for (int i=PROP; i<TEX64; i++)  IdentityMatrix(&Sprite64[i].World);
	for (int i=FIRE; i<TEX32; i++)  IdentityMatrix(&Sprite32[i].World);
	IdentityMatrix(&Sprite96[JET].World);
	IdentityMatrix(&Sprite94[SUB].World);


	for (int i=PROP; i<TEX64; i++)			//builds up prop plane, lagoon,
	{						//island and vulcano.
	  Sprite64[i].Pos.v[0] = p[i+1][0];
	  Sprite64[i].Pos.v[1] = p[i+1][1];
	  Sprite64[i].Pos.v[2] = z[i+1];
	  Sprite64[i].Pos.v[3] = 1.0f;
	  Sprite64[i].w = 64;
	  Sprite64[i].h = 64;
	  // Untransformed Sprite positions
	  // Top Left
	  Sprite64[i].TL.v[0] = -(float)(Sprite64[i].w/2);;
	  Sprite64[i].TL.v[1] = -(float)(Sprite64[i].h/2);
	  Sprite64[i].TL.v[2] = 0.0f;
	  Sprite64[i].TL.v[3] = 1.0f;
	  // Bottom Left
	  Sprite64[i].BL.v[0] = -(float)(Sprite64[i].w/2);
	  Sprite64[i].BL.v[1] =  (float)(Sprite64[i].h/2);
	  Sprite64[i].BL.v[2] = 0.0f;
	  Sprite64[i].BL.v[3] = 1.0f;
	  // Top Right
	  Sprite64[i].TR.v[0] =  (float)(Sprite64[i].w/2);
	  Sprite64[i].TR.v[1] = -(float)(Sprite64[i].h/2);
	  Sprite64[i].TR.v[2] = 0.0f;
	  Sprite64[i].TR.v[3] = 1.0f;
	  // Bottom Right
	  Sprite64[i].BR.v[0] = (float)(Sprite64[i].w/2);
	  Sprite64[i].BR.v[1] = (float)(Sprite64[i].h/2);
	  Sprite64[i].BR.v[2] = 0.0f;
	  Sprite64[i].BR.v[3] = 1.0f;
	  // UV Texture offsets
	  Sprite64[i].u = offset[i][0];
	  Sprite64[i].v = offset[i][1];
	  // Colour
	  Sprite64[i].r = 0x80;
	  Sprite64[i].g = 0x80;
	  Sprite64[i].b = 0x80;
	  Sprite64[i].a = 0x80;
	  // Animation parameters
	  Sprite64[i].FCountMax = 5;
	  Sprite64[i].FCount = 0;
	  Sprite64[i].NFrames = 3;
	  // Rotation angle
	  Sprite64[i].Rot = 0.0f;
	 }

	Sprite96[JET].Pos.v[0] = p[0][0];		//builds up jet plane
	Sprite96[JET].Pos.v[1] = p[0][1];
	Sprite96[JET].Pos.v[2] = z[0];
	Sprite96[JET].Pos.v[3] = 1.0f;
	Sprite96[JET].w = 96;
	Sprite96[JET].h = 96;
	// Top Left
	Sprite96[JET].TL.v[0] = -(float)(Sprite96[JET].w/2);;
	Sprite96[JET].TL.v[1] = -(float)(Sprite96[JET].h/2);
	Sprite96[JET].TL.v[2] = 0.0f;
	Sprite96[JET].TL.v[3] = 1.0f;
	// Bottom Left
	Sprite96[JET].BL.v[0] = -(float)(Sprite96[JET].w/2);
	Sprite96[JET].BL.v[1] =  (float)(Sprite96[JET].h/2);
	Sprite96[JET].BL.v[2] = 0.0f;
	Sprite96[JET].BL.v[3] = 1.0f;
	// Top Right
	Sprite96[JET].TR.v[0] =  (float)(Sprite96[JET].w/2);
	Sprite96[JET].TR.v[1] = -(float)(Sprite96[JET].h/2);
	Sprite96[JET].TR.v[2] = 0.0f;
	Sprite96[JET].TR.v[3] = 1.0f;
	// Bottom Right
	Sprite96[JET].BR.v[0] = (float)(Sprite96[JET].w/2);
	Sprite96[JET].BR.v[1] = (float)(Sprite96[JET].h/2);
	Sprite96[JET].BR.v[2] = 0.0f;
	Sprite96[JET].BR.v[3] = 1.0f;
	// UV Texture offsets
	Sprite96[JET].u = 161;
	Sprite96[JET].v = 64;
	// Colour
	Sprite96[JET].r = 0x80;
	Sprite96[JET].g = 0x80;
	Sprite96[JET].b = 0x80;
	Sprite96[JET].a = 0x80;
	// Animation parameters
	Sprite96[JET].FCountMax = 5;
	Sprite96[JET].FCount = 0;
	Sprite96[JET].NFrames = 1;
	// Rotation angle
	Sprite96[JET].Rot = 0.0f;

	for (int i=FIRE; i<TEX32; i++)
	{
	Sprite32[i].Pos.v[0] = 1.0f;		//builds up fire & explosion sprites
	Sprite32[i].Pos.v[1] = 1.0f;		//initial coordinates and z value are irrilevant.
	Sprite32[i].Pos.v[2] = 1.0f;		//Will match shooter/destroyed ship's ones.
	Sprite32[i].Pos.v[3] = 1.0f;
	Sprite32[i].w = 32;
	Sprite32[i].h = 25;
	// Top Left
	Sprite32[i].TL.v[0] = -(float)(Sprite32[i].w/2);;
	Sprite32[i].TL.v[1] = -(float)(Sprite32[i].h/2);
	Sprite32[i].TL.v[2] = 0.0f;
	Sprite32[i].TL.v[3] = 1.0f;
	// Bottom Left
	Sprite32[i].BL.v[0] = -(float)(Sprite32[i].w/2);;
	Sprite32[i].BL.v[1] =  (float)(Sprite32[i].h/2);
	Sprite32[i].BL.v[2] = 0.0f;
	Sprite32[i].BL.v[3] = 1.0f;
	// Top Right
	Sprite32[i].TR.v[0] =  (float)(Sprite32[i].w/2);;
	Sprite32[i].TR.v[1] = -(float)(Sprite32[i].h/2);
	Sprite32[i].TR.v[2] = 0.0f;
	Sprite32[i].TR.v[3] = 1.0f;
	// Bottom Right
	Sprite32[i].BR.v[0] = (float)(Sprite32[i].w/2);;
	Sprite32[i].BR.v[1] = (float)(Sprite32[i].h/2);
	Sprite32[i].BR.v[2] = 0.0f;
	Sprite32[i].BR.v[3] = 1.0f;
	// Colour
	Sprite32[i].r = 0x80;
	Sprite32[i].g = 0x80;
	Sprite32[i].b = 0x80;
	Sprite32[i].a = 0x80;
	// Rotation angle
	Sprite32[i].Rot = 0.0f;
	}
	// UV Texture offsets
	Sprite32[FIRE].u = 196;
	Sprite32[FIRE].v = 0;
	Sprite32[EXPLOSION].u = 0;
	Sprite32[EXPLOSION].v = 230;
	// Animation parameters
	Sprite32[FIRE].FCountMax = 5;
	Sprite32[FIRE].FCount = 0;
	Sprite32[FIRE].NFrames = 1;
	Sprite32[EXPLOSION].FCountMax = 30;
	Sprite32[EXPLOSION].FCount = 0;
	Sprite32[EXPLOSION].NFrames = 6;

	Sprite94[SUB].Pos.v[0] = p[5][0];		//builds up submarine
	Sprite94[SUB].Pos.v[1] = p[5][1];
	Sprite94[SUB].Pos.v[2] = z[5];
	Sprite94[SUB].Pos.v[3] = 1.0f;
	Sprite94[SUB].w = 32;
	Sprite94[SUB].h = 94;
	// Top Left
	Sprite94[SUB].TL.v[0] = -(float)(Sprite94[SUB].w/2);;
	Sprite94[SUB].TL.v[1] = -(float)(Sprite94[SUB].h/2);
	Sprite94[SUB].TL.v[2] = 0.0f;
	Sprite94[SUB].TL.v[3] = 1.0f;
	// Bottom Left
	Sprite94[SUB].BL.v[0] = -(float)(Sprite94[SUB].w/2);
	Sprite94[SUB].BL.v[1] =  (float)(Sprite94[SUB].h/2);
	Sprite94[SUB].BL.v[2] = 0.0f;
	Sprite94[SUB].BL.v[3] = 1.0f;
	// Top Right
	Sprite94[SUB].TR.v[0] =  (float)(Sprite94[SUB].w/2);
	Sprite94[SUB].TR.v[1] = -(float)(Sprite94[SUB].h/2);
	Sprite94[SUB].TR.v[2] = 0.0f;
	Sprite94[SUB].TR.v[3] = 1.0f;
	// Bottom Right
	Sprite94[SUB].BR.v[0] = (float)(Sprite94[SUB].w/2);
	Sprite94[SUB].BR.v[1] = (float)(Sprite94[SUB].h/2);
	Sprite94[SUB].BR.v[2] = 0.0f;
	Sprite94[SUB].BR.v[3] = 1.0f;
	// UV Texture offsets
	Sprite94[SUB].u = 0;
	Sprite94[SUB].v = 64;
	// Colour
	Sprite94[SUB].r = 0x80;
	Sprite94[SUB].g = 0x80;
	Sprite94[SUB].b = 0x80;
	Sprite94[SUB].a = 0x80;
	// Animation parameters
	Sprite94[SUB].FCountMax = 35;
	Sprite94[SUB].FCount = 0;
	Sprite94[SUB].NFrames = 5;
	// Rotation angle
	Sprite94[SUB].Rot = 0.0f;

	//get the required register values
	// Clear the register to zero
	madr.i32 = 0;
	// Set the address field to the physical address of the allocated mem
	madr.s.ADDR = sps2GetPhysicalAddress(pPrimitiveMem, pMemCached);

	// The channel control register
	chcr.i32 = 0;
	chcr.s.MOD = CHCR_MOD_NORMAL;			// Normal mode transfer
	chcr.s.STR = 1;					// Start the transfer

	// We want to send the packet to the GS every frame
	do
	{
		Frame++;

		//update the pad structure
		pad_update (PAD_0 | PAD_1);

		//reset the pointer to the start of primitive memory
		pPrimitiveMem = &pcMemory[PRIMITIVEMEM_4K];

		//clear the screen to the specified colour
		sps2UScreenClear(0x00, 0x43, 0xab);	// SKY BLUE (00 67 171 dec)

		DoControls(&Sprite96[JET], &Sprite64[PROP]);

		if(pad[0].buttons & PAD_CROSS) MoveSprite(&Sprite96[JET], -pad[0].pressures[PAD_PCROSS] * 5.0f, YAXIS);
 		else MoveSprite(&Sprite96[JET], -3.0f, YAXIS);
 		if(pad[1].buttons & PAD_CROSS) MoveSprite(&Sprite64[PROP], -pad[1].pressures[PAD_PCROSS] * 5.0f, YAXIS);
 		else MoveSprite(&Sprite64[PROP], -3.0f, YAXIS);

		//fire control
 		if(pad[0].buttons & PAD_CIRCLE) {
				  SetFire(&Sprite32[FIRE], &Sprite96[JET]);
				  JetFire=1;
				  }
 		else if(pad[1].buttons & PAD_CIRCLE) {
 				       SetFire(&Sprite32[FIRE], &Sprite64[PROP]);
				       PropFire=1;}

		CheckPlayground(&Sprite96[JET], &Sprite64[PROP]);

		if(Fire) MoveSprite(&Sprite32[FIRE], -8.0f, YAXIS);
		//if sprite exits the screen it is destroyed
		if((Sprite32[FIRE].Pos.v[0]>320.0f) || (Sprite32[FIRE].Pos.v[0]<-320.0f) ||
		   (Sprite32[FIRE].Pos.v[1]>256.0f) || (Sprite32[FIRE].Pos.v[1]<-256.0f)) Fire=0;

		//collision detection and response  - NOT WORKING YET.. HELP NEEDED!
		/*if((Collision(&Sprite32[FIRE], &Sprite96[JET])) && PropFire)
		{
		 Sprite32[EXPLOSION].Pos.v[0]=Sprite96[JET].Pos.v[0];
		 Sprite32[EXPLOSION].Pos.v[1]=Sprite96[JET].Pos.v[1];
		 Sprite32[EXPLOSION].Rot=Sprite96[JET].Rot;
		 JetDead = 1;
		 }
		 else if((Collision(&Sprite32[FIRE], &Sprite64[PROP])) && JetFire)
		 	{
		 	  Sprite32[EXPLOSION].Pos.v[0]=Sprite64[PROP].Pos.v[0];
		 	  Sprite32[EXPLOSION].Pos.v[1]=Sprite64[PROP].Pos.v[1];
		 	  Sprite32[EXPLOSION].Rot=Sprite64[PROP].Rot;
			  PropDead = 1;
		 	}*/

		if(!PropDead) AnimateSprite(&Sprite64[PROP]);
		AnimateSub(&Sprite94[SUB]);
		if(Explosion) AnimateSprite(&Sprite32[EXPLOSION]);
		if(Sprite32[EXPLOSION].u>(Sprite32[EXPLOSION].NFrames-2)*Sprite32[EXPLOSION].w)  //end game
		{
		  if(PropDead) GameOver(1);
		  else if(JetDead) GameOver(0);
		  exit(1);
		 }

		//build the GS packet
  		 for (int i=LAGOON; i<TEX64; i++) BuildSprite2D(&Sprite64[i], pPrimitiveMem);
		 BuildSprite2D(&Sprite94[SUB], pPrimitiveMem);
		 if(!Explosion)
		 {
		   BuildSprite2D(&Sprite96[JET], pPrimitiveMem);
		   BuildSprite2D(&Sprite64[PROP], pPrimitiveMem);
		   if(Fire) BuildSprite2D(&Sprite32[FIRE], pPrimitiveMem);
		  }
		  else
		  {
		    BuildSprite2D(&Sprite32[EXPLOSION], pPrimitiveMem);
		    if(JetDead) BuildSprite2D(&Sprite64[PROP], pPrimitiveMem);
		    if(PropDead) BuildSprite2D(&Sprite96[JET], pPrimitiveMem);
		   }

		//calculate how many qwords have been written
		int iQWC = ((int)pPrimitiveMem - (int)&pcMemory[PRIMITIVEMEM_4K]) >> 4;

		// Assert that we haven't written more data than we allocated
		assert(iQWC < (4096 >> 4));

		// Important: Make sure all the data is written to the physical memory
		sps2FlushCache(iSPS2Desc);

		// DMA Channel 2 is the one we want, it goes to the GS
		// Amount of data to be transmitted
		*EE_D2_QWC = iQWC;

		// Set the memory address register
		*EE_D2_MADR = madr.i32;

		// Set the register to start the transfer
		*EE_D2_CHCR = chcr.i32;

		// Wait for it to complete
		sps2WaitForDMA(2, iSPS2Desc);

		// Display the buffer we were just drawing to
		sps2UScreenSwap();

	}while(!((pad[0].buttons & PAD_START)&&(pad[0].buttons & PAD_SELECT)));

	pad_cleanup (PAD_0 | PAD_1);

	// Shutdown the screen
	sps2UScreenShutdown();

	// Release SPS2
	sps2Release(iSPS2Desc);

	return 0;
}


//move Sprite in the direction it is pointing
void MoveSprite(PS2Sprite_t * pSP, float Delta, int Axis)
{
	//this moves the Sprite along its local "Axis" axis
	// by an amount Delta
	pSP->Pos.v[0] += pSP->World.m[0][Axis]* Delta;
	pSP->Pos.v[1] += pSP->World.m[1][Axis]* Delta;
	pSP->Pos.v[2] += pSP->World.m[2][Axis]* Delta;
}


//some very simple animation - changes the frame every time the counter reaches FCountMax
void AnimateSprite(PS2Sprite_t * pSP)
{
	if((pSP->FCount++)%pSP->FCountMax == 0)
	{
		pSP->u += pSP->w+2;	//"2" is the width of frame separator
		if(pSP->u > (pSP->NFrames - 1) * pSP->w) pSP->u = 0;
	}
}


void AnimateSub(PS2Sprite_t * pSP)
{
	if((pSP->FCount++)%pSP->FCountMax == 0)
	{
		pSP->u += (pSP->w+2)*Loop;	//"Loop" controls sub movement up&down
		if((pSP->u == 0) || (pSP->u > (pSP->NFrames - 1) * pSP->w)) Loop *=-1;
	}
}


//Game intro - initial development stage. Need to load "intro1945.bmp" texture as background.
//see tex2d.cpp for details. Need to extend memory usage in the define section
//e.g. #define TEXTUREMEM_512K macro
void TitleScreen(void)
{
  do
    {
      pad_update(PAD_0);
      sps2UScreenClear(0,0,0);
      sps2UPrintfSetZ(255);		//put credits in end game routine
      sps2UPrintf(0,"\377FF0000\376(200,130)Air War 1945\n");
      sps2UPrintf(0,"\377FFFFFF\376(200,190)by g0blin\n");
      sps2UPrintf(0,"\376(200,240)Engine Core by HFortuna - Artwork by Ari Feldman\n");
      sps2UPrintf(0,"\376(200,270)Use D-buttons to turn Left and Right\n");
      sps2UPrintf(0,"\376(200,290)Press X to increase speed\n");
      sps2UPrintf(0,"\376(200,310)All Buttons are pressure sensitive\n");
      sps2UPrintf(0,"\376(200,350)- Press X on PAD 1 to start -\n");
      sps2UPrintf(0,"\376(200,370)- Press START&SELECT on PAD 1 to quit game -\n");
      sps2UPrintfRender();
      sps2UScreenSwap();
     }
   while (!(pad[0].buttons & PAD_CROSS));
  }

  //End game routine
void GameOver(int lives)
{
  do
    {
      pad_update (PAD_0 | PAD_1);
      sps2UScreenClear(0,0,0);
      sps2UPrintfSetZ(255);
      sps2UPrintf(0,"\377FF0000\376(200,130)GAME OVER !\n");
      if (lives == 0 ) sps2UPrintf(0,"\376(200,170)Player Two wins !\n");
      else sps2UPrintf(0,"\376(200,170)Player One wins !\n");
      sps2UPrintf(0,"\377FFFFFF\376(200,210)Thank you for playing.\n");
      sps2UPrintfRender();
      sps2UScreenSwap();
    }
  while (!(pad[0].buttons & PAD_CROSS));
}


bool Collision(PS2Sprite_t * pSP1, PS2Sprite_t * pSP2)
//collision detection using bounding boxes
{
  /*if(int(((pSP1->Pos.v[0] + pSP1->w) < pSP2->Pos.v[0]) ||
         (pSP1->Pos.v[0] > (pSP2->Pos.v[0] + pSP2->w)) ||
	 ((pSP1->Pos.v[1] + pSP1->h) < pSP2->Pos.v[1]) ||
	 (pSP1->Pos.v[1] > (pSP2->Pos.v[1] + pSP2->h)))) return false; //no collision
  else { Explosion=1;
         return true;}	 	//collision */
//collision detection using bounding circles
  float a,b,c;
  a = pSP2->Pos.v[0] - pSP1->Pos.v[0];
  b = pSP2->Pos.v[1] - pSP1->Pos.v[1];
  c = (float)(pSP2->w/2 + pSP1->w/2);
  if(a*a + b*b < c*c) { Explosion = 1;		//collision
  			return true;}
  else return false;				//no collision
}


void DoControls(PS2Sprite_t * pSP1, PS2Sprite_t * pSP2)
{
 //rotates sprites
 if(pad[0].buttons & PAD_RIGHT) pSP1->Rot += pad[0].pressures[PAD_PRIGHT] * 0.2f;
 if(pad[0].buttons & PAD_LEFT) pSP1->Rot -= pad[0].pressures[PAD_PLEFT] * 0.2f;
 if(pad[1].buttons & PAD_RIGHT) pSP2->Rot += pad[1].pressures[PAD_PRIGHT] * 0.2f;
 if(pad[1].buttons & PAD_LEFT) pSP2->Rot -= pad[1].pressures[PAD_PLEFT] * 0.2f;
 //moves sprites automatically unless players take control
 /*
 if(pad[0].buttons & PAD_CROSS) MoveSprite(&Sprite96[JET], -pad[0].pressures[PAD_PCROSS] * 5.0f, YAXIS);
 else MoveSprite(&Sprite96[JET], -3.0f, YAXIS);
 if(pad[1].buttons & PAD_CROSS) MoveSprite(&Sprite64[PROP], -pad[1].pressures[PAD_PCROSS] * 5.0f, YAXIS);
 else MoveSprite(&Sprite64[PROP], -3.0f, YAXIS);
 //fire control
 if(pad[0].buttons & PAD_CIRCLE) {
				  SetFire(&Sprite32[FIRE], &Sprite96[JET]);
				  JetFire=1;
				  }
 else if(pad[1].buttons & PAD_CIRCLE) {
 				       SetFire(&Sprite32[FIRE], &Sprite64[PROP]);
				       PropFire=1;
				       }*/
 }


 void CheckPlayground(PS2Sprite_t * pSP1, PS2Sprite_t * pSP2)
 {
  //if sprites exit the screen, they reenter on the other side
  if((pSP1->Pos.v[0]>320.0f) || (pSP1->Pos.v[0]<-320.0f)) pSP1->Pos.v[0]*= -1;
  if((pSP1->Pos.v[1]>256.0f) || (pSP1->Pos.v[1]<-256.0f)) pSP1->Pos.v[1]*= -1;
  if((pSP2->Pos.v[0]>320.0f) || (pSP2->Pos.v[0]<-320.0f)) pSP2->Pos.v[0]*= -1;
  if((pSP2->Pos.v[1]>256.0f) || (pSP2->Pos.v[1]<-256.0f)) pSP2->Pos.v[1]*= -1;
  }


//set up fire sprite
 void SetFire(PS2Sprite_t * pSP1, PS2Sprite_t * pSP2)
 {
  pSP1->Pos.v[0]=pSP2->Pos.v[0];
  pSP1->Pos.v[1]=pSP2->Pos.v[1];
  pSP1->Rot=pSP2->Rot;
  Fire=1;
  }
